/*******************************************************************************
Copyright © 2023 Sebastian Roith

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************/

(function(window) {

const LEVELS = [
    {
        task: '<p>In jeder der nachfolgenden Aufgaben geht es darum, dass der Roboter eine Umrandung seiner Welt legt (oder einen Teil davon). Als Welt zu dieser Aufgabe siehst du eine fertige Umrandung. Schaue sie dir an, damit du verstehst, was immer mit Umrandung gemeint ist.</p><p>Verwende bei allen Aufgaben nur Anweisungen, Sequenzen und Wiederholungen mit fester Anzahl. Deine Programme müssen so kurz wie möglich sein. Verwende also stets eine Wiederholung mit fester Anzahl, wenn du dadurch Sequenzen abkürzen kannst.</p>',
        world: '015501010101010100000001010000000101000000010101010101003',
        code: '# Ab Aufgabe 2 darfst du programmieren…\n',
        readonly: true,
        checkCode: [
            CodeCheckers.reject(0, 'Programme testest du erst ab Aufgabe 2')
        ],
        questionWrongFine: 1,
        question: {
            html: 'Was ist bei den nachfolgenden Aufgaben wichtig?',
            response: 'Lies die oben stehende Aufgabe erneut!',
            choices: [
                { html: "Selbstdefinierte Anweisungen sind nicht erlaubt", correct: true },
                { html: "Wiederholungen mit fester Anzahl sind nicht erlaubt" },
                { html: "(Vordefinierte) Anweisungen sind nicht erlaubt" },
                { html: "Sequenzen sind nicht erlaubt" },
                { html: "Der Roboter soll die Umrandung entfernen" },
                { html: "Statt Wiederholungen mit fester Anzahl sollen möglichst Sequenzen verwendet werden" },
                { html: "Wo immer sinnvoll möglich sollen Wiederholungen mit fester Anzahl  verwendet werden", correct: true },
                { html: "Der Roboter soll eine hier gezeigte Umrandung bauen", correct: true }
            ]
        }
    },
    {
        task: 'Schreibe nun ein Programm, dass den Roboter die erste Seite der Umrandung bauen lässt: Er soll auf die Felder vor ihm in Blickrichtung je ein Plättchen legen.',
        world: '015500000000000000000000000000000000000000000000000000003',
        code: '# Umrandung bauen\n',
        locked: true,
        checkCode: [
            CodeCheckers.disallowIfAndWhile(),
            CodeCheckers.disallowDefinition(),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.or(
                CodeCheckers.ensureMain(1, 'WIEDERHOLE 4 MAL\n  plus\n  vor', true),
                CodeCheckers.ensureMain(1, 'plus\nWIEDERHOLE 4 MAL\n  vor\n  plus', true),
                CodeCheckers.ensureMain(1, 'WIEDERHOLE 4 MAL\n  plus\n  vor\nplus', true)
            ),
            CodeCheckers.accept(true)
        ]
    },
    {
        task: 'Nun soll der Roboter die gesamte Umrandung bauen. Dazu musst du dein bisherigen Programm nur ein wenig anpassen und dann entsprechend oft wiederholen…',
        world: '015500000000000000000000000000000000000000000000000000003',
        code: '# Umrandung bauen\nWIEDERHOLE 4 MAL\n    plus\n    vor\nplus\n',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.disallowIfAndWhile(),
            CodeCheckers.disallowDefinition(),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.ensureMain(1, 'WIEDERHOLE 4 MAL\n    WIEDERHOLE 4 MAL\n        plus\n        vor\n    linksrum', true),
            CodeCheckers.accept(true)
        ]
    },
    {
        task: 'Schaffst du es, dass die Umrandung drei Plättchen hoch ist? Dabei soll der Roboter jedes Feld höchstens einmal betreten und dabei gleich alle drei Plättchen hinlegen.',
        world: '015500000000000000000000000000000000000000000000000000003',
        code: '# Umrandung bauen\nWIEDERHOLE 4 MAL\n    WIEDERHOLE 4 MAL\n        plus\n        vor\n    linksrum\n',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.disallowIfAndWhile(),
            CodeCheckers.disallowDefinition(),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.ensureMain(1, 'WIEDERHOLE 4 MAL\n    WIEDERHOLE 4 MAL\n        WIEDERHOLE 3 MAL\n            plus\n        vor\n    linksrum', true),
            CodeCheckers.accept(true)
        ]
    },
    {
        task: '<p>Planänderung: Der Roboter soll zwar weiterhin eine drei Plättchen hohe Umrandung bauen. Jedoch sollen die Schichten nacheinander aufgetragen werden. Also erst nur ein Plättchen auf jedes äußere Feld, dann erst das zweite Plättchen auf jedes äußere Feld, dann erst das dritte.</p><p>Ändere dein Programm entsprechend ab:</p>',
        world: '015500000000000000000000000000000000000000000000000000003',
        code: '# Umrandung bauen\nWIEDERHOLE 4 MAL\n    WIEDERHOLE 4 MAL\n        WIEDERHOLE 3 MAL\n            plus\n        vor\n    linksrum\n',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.disallowIfAndWhile(),
            CodeCheckers.disallowDefinition(),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.ensureMain(1, 'WIEDERHOLE 3 MAL\n    WIEDERHOLE 4 MAL\n        WIEDERHOLE 4 MAL\n            plus\n            vor\n        linksrum', true),
            CodeCheckers.accept(true)
        ]
    },
    {
        task: 'Jetzt hat sich die Größe der Roboterwelt verändert. Passe dein Programm entsprechend an:',
        world: '0164000000000000000000000000000000000000000000000000003',
        code: '# Umrandung bauen\nWIEDERHOLE 3 MAL\n    WIEDERHOLE 4 MAL\n        WIEDERHOLE 4 MAL\n            plus\n            vor\n        linksrum\n',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.disallowIfAndWhile(),
            CodeCheckers.disallowDefinition(),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.or(
                CodeCheckers.ensureMain(1, 'WIEDERHOLE 3 MAL\n    WIEDERHOLE 2 MAL\n        WIEDERHOLE 5 MAL\n            plus\n            vor\n        linksrum\n        WIEDERHOLE 3 MAL\n            plus\n            vor\n        linksrum', true),
                CodeCheckers.ensureMain(1, 'WIEDERHOLE 3 MAL\n    WIEDERHOLE 2 MAL\n        WIEDERHOLE 2 MAL\n            plus\n            vor\n        WIEDERHOLE 2 MAL\n            WIEDERHOLE 3 MAL\n                plus\n                vor\n            linksrum', true)
            ),
            CodeCheckers.accept(true)
        ]
    },
    {
        task: 'Schaffst du es auch, dass sich der Roboter an den Ecken statt linksherum rechtsherum dreht?',
        world: '0164000000000000000000000000000000000000000000000000000',
        code: '# Umrandung bauen\nWIEDERHOLE 3 MAL\n    WIEDERHOLE 4 MAL\n        WIEDERHOLE 5 MAL\n            plus\n            vor\n        linksrum\n        WIEDERHOLE 3 MAL\n            plus\n            vor\n        linksrum\n',
        keepCode: true,
        locked: true,
        checkCode: [
            CodeCheckers.disallowIfAndWhile(),
            CodeCheckers.disallowDefinition(),
            CodeCheckers.accept()
        ],
        checkTermination: [
            CodeCheckers.acceptUserAbort(1),
            CodeCheckers.allowOnlyDone(1),
            CodeCheckers.ensureMain(1, 'WIEDERHOLE 3 MAL\n    WIEDERHOLE 2 MAL\n        WIEDERHOLE 3 MAL\n            plus\n            vor\n        WIEDERHOLE 3 MAL\n            linksrum\n        WIEDERHOLE 5 MAL\n            plus\n            vor\n        WIEDERHOLE 3 MAL\n            linksrum', true),
            CodeCheckers.accept(true)
        ]
    }
];

window.roboterCodingTask = {
    levels: LEVELS
};
})(window);
